/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+                                                                             +
+  GetKernel32Calls - access LoadLibraryA() and GetProcAddress()              +
+                                                                             +
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

#include <ntimage.h>

#define IMAGE_DOS_SIGNATURE                 0x5A4D      // MZ
#define IMAGE_NT_SIGNATURE                  0x00004550  // PE00

DWORD getHashA(char* str)
{
	DWORD hash;
	hash = 0;
	while(*str != '\0')
	{
		hash = hash + (*(unsigned char*)str|0x60);
		hash = hash << 1;
		str = str + 1;
	}
	return(hash);
}/*end getHash()------------------------------------------------------------*/

DWORD getHashW(WCHAR* str)
{
	DWORD hash;
	hash = 0;
	while(*str != '\0')
	{
		hash = hash + *str;
		str = str + 1;
	}
	return(hash);
}/*end getHash()------------------------------------------------------------*/

BOOLEAN walkExportList(DWORD baseAddress, DWORD* functionPtr, DWORD hashValue)
{

    PIMAGE_DOS_HEADER			dosHeader;
    PIMAGE_NT_HEADERS			peHeader;
    IMAGE_OPTIONAL_HEADER		optionalHeader;
    IMAGE_DATA_DIRECTORY		dataDirectory;
    DWORD						descriptorStartRVA;
    PIMAGE_EXPORT_DIRECTORY		exportDirectory;
    unsigned int				index;
	char						*dllName;
	DWORD*						routineNames;	
	DWORD*						rvas;			
	WORD*						ordinals;	
    dosHeader = (PIMAGE_DOS_HEADER)baseAddress;
    if(((*dosHeader).e_magic)!=IMAGE_DOS_SIGNATURE)
    { 
        return(FALSE);
    }
    peHeader = (PIMAGE_NT_HEADERS)((DWORD)baseAddress + (*dosHeader).e_lfanew);
    if(((*peHeader).Signature)!=IMAGE_NT_SIGNATURE)
    { 
        return(FALSE);
    }
    optionalHeader = (*peHeader).OptionalHeader;
    if((optionalHeader.Magic)!=0x10B)
    {
        return(FALSE);
    }
    dataDirectory = (optionalHeader).DataDirectory[IMAGE_DIRECTORY_ENTRY_EXPORT];
    descriptorStartRVA = dataDirectory.VirtualAddress;
    exportDirectory = (PIMAGE_EXPORT_DIRECTORY)(descriptorStartRVA + (DWORD)baseAddress);
	dllName = (char*)((*exportDirectory).Name + (DWORD)baseAddress);
	routineNames = (DWORD*)((*exportDirectory).AddressOfNames + (DWORD)baseAddress);
	rvas = (DWORD*)((*exportDirectory).AddressOfFunctions + (DWORD)baseAddress);
	ordinals = (WORD*)((*exportDirectory).AddressOfNameOrdinals + (DWORD)baseAddress);
	for(index=0;index<(*exportDirectory).NumberOfFunctions;index++)
    {
		DWORD j;
		DWORD name;
		DWORD entryPointRVA;
		DWORD entryPointActual;
		entryPointRVA		= rvas[index];
		if(entryPointRVA==0){ continue; }
		for(j=0;j<(*exportDirectory).NumberOfFunctions;j++)
		{
			if(ordinals[j]==index)
			{
				name = (DWORD)(routineNames[j] + (DWORD)baseAddress);
				entryPointActual = (DWORD)(rvas[index] + (DWORD)baseAddress);
				if(getHashA((char*)name)==hashValue)
				{
					*functionPtr = entryPointActual;
				}
			}
		}
    }
    return(TRUE);
}/*end walkExportList()------------------------------------------------------*/

#define IA32_SYSENTER_EIP	0x176

typedef struct _MSR
{
	DWORD loValue;	
	DWORD hiValue;	
}MSR;

void getMSR(DWORD regAddress, MSR* msr)
{
	DWORD loValue;
	DWORD hiValue;
	__asm
	{
		MOV ECX,regAddress
		rdmsr
		MOV hiValue, edx
		MOV loValue, eax
	}
	(*msr).loValue = loValue;	
	(*msr).hiValue = hiValue;
	
	return;
}/*end getMSR()--------------------------------------------------------------*/

BOOLEAN GetNTLibAddress(DWORD* routineAddress, DWORD hash)
{
	MSR msr;
	BYTE *bytePtr;
	DWORD baseAddress;
	getMSR(IA32_SYSENTER_EIP,&msr);
	bytePtr = (BYTE*)(msr.loValue);
	while((bytePtr[0]!=0x4D)||(bytePtr[1]!=0x5A)||(bytePtr[2]!=0x90))
	{
		bytePtr--;
	}
	baseAddress = (DWORD)bytePtr;
	return(walkExportList(baseAddress, routineAddress, hash));
}/*end GetNTLibAddress()-----------------------------------------------------*/